﻿using System;
using System.IO;
using System.Runtime.InteropServices;
using System.Text;
using System.Drawing.Drawing2D;
using System.Drawing;
using System.Collections.Generic;

/// <summary>
/// Extensions.
/// </summary>
public static class MerthsoftExtensions {
	/// <summary>
	/// Read an array of elements from a <see cref="BinaryReader"/>.
	/// </summary>
	/// <typeparam name="T">The type of element to read.</typeparam>
	/// <param name="reader">The <see cref="BinaryReader"/> to read from.</param>
	/// <param name="count">The number of elements to read.</param>
	/// <returns>An array of elements of type <typeparamref name="T"/>.</returns>
	public static T[] ReadArray<T>(this BinaryReader reader, int count) {
		var RawData = reader.ReadBytes(count * Marshal.SizeOf(typeof(T)));
		var Result = new T[count];
		var PinnedResult = GCHandle.Alloc(Result, GCHandleType.Pinned);
		try {
			Marshal.Copy(RawData, 0, PinnedResult.AddrOfPinnedObject(), RawData.Length);
		} finally {
			PinnedResult.Free();
		}
		return Result;
	}

	public static byte[] ToByteArray(this int i, bool bigEndian = false) {
		if (!bigEndian) {
			return BitConverter.GetBytes(i);
		} else {
			byte[] b = BitConverter.GetBytes(i);
			Array.Reverse(b);
			return b;
		}
	}

	public static byte[] ToByteArray(this short s, bool bigEndian = false) {
		if (!bigEndian) {
			return BitConverter.GetBytes(s);
		} else {
			byte[] b = BitConverter.GetBytes(s);
			Array.Reverse(b);
			return b;
		}
	}

	public static ushort ToUShort(this byte[] b, int index, bool bigEndian = false) {
		if (!bigEndian) {
			return BitConverter.ToUInt16(b, index);
		} else {
			byte[] sub = b.SubArray(index, 2);
			Array.Reverse(sub);
			return BitConverter.ToUInt16(sub, 0);
		}
	}

	public static uint ToUInt(this byte[] b, int index, bool bigEndian = false) {
		if (!bigEndian) {
			return BitConverter.ToUInt32(b, index);
		} else {
			byte[] sub = b.SubArray(index, 2);
			Array.Reverse(sub);
			return BitConverter.ToUInt32(sub, 0);
		}
	}

	/// <summary>
	/// Gets a subset of the given length starting at the given index.
	/// </summary>
	/// <typeparam name="T">The type of the array.</typeparam>
	/// <param name="data">The array you want to get the subset from.</param>
	/// <param name="index">The start index to copy from.</param>
	/// <param name="length">The length of the array to return.</param>
	/// <returns>A subset from the original array.</returns>
	public static T[] SubArray<T>(this T[] data, int index, int length) {
		T[] result = new T[length];
		Array.Copy(data, index, result, 0, length);
		return result;
	}

	/// <summary>
	/// Calculates the checksum of a byte array.
	/// </summary>
	/// <param name="buffer">The array of bytes on which to calculate the checksum.</param>
	/// <returns>The last two bytes (as a short) of the sum of all the bytes in the buffer.</returns>
	public static short Checksum(this byte[] buffer) {
		Int64 sum = 0;
		foreach (byte b in buffer) {
			sum += b;
		}
		return (short)(sum % 65536);
	}

	/// <summary>
	/// Gets the bytes that make up the givens short.
	/// </summary>
	/// <param name="s">The short to get the bytes from.</param>
	/// <returns>The two-byte array that this short is made up of.</returns>
	public static byte[] GetBytes(this short s) {
		return BitConverter.GetBytes(s);
	}

	/// <summary>
	/// Converts this string into an array of ASCII bytes.
	/// </summary>
	/// <param name="s">The string to convert</param>
	/// <param name="arrayLen">How long the returned array should be. If it's longer than the string, it will pad the string right with the pad char.</param>
	/// <param name="padChar">The character used for padding the string.</param>
	/// <returns>An array of bytes representing the string in ASCII.</returns>
	public static byte[] ToByteArray(this string s, int arrayLen = -1, char padChar = '\0') {
		if (arrayLen == -1)
			arrayLen = s.Length;
		byte[] arr = new byte[arrayLen];
		ASCIIEncoding.ASCII.GetBytes(s.PadRight(arrayLen, padChar), 0, arrayLen, arr, 0);
		return arr;
	}

	public static byte[] ToPackedByteArray(this string s) {
		int NumberChars = s.Length;
		byte[] bytes = new byte[NumberChars / 2];
		for (int i = 0; i < NumberChars; i += 2)
			bytes[i / 2] = Convert.ToByte(s.Substring(i, 2), 16);
		return bytes;
	}

	public static string AsString(this byte[] array) {
		StringBuilder sb = new StringBuilder();
		foreach (byte b in array) {
			sb.Append(b.ToString("X2"));
		}

		return sb.ToString();
	}

	public static string ToAsciiString(this byte[] b) {
		return ASCIIEncoding.ASCII.GetString(b);
	}

	public static string ToUtf8String(this byte[] b) {
		return UTF8Encoding.UTF8.GetString(b);
	}

	public static void FloodFill(this Bitmap b, Pen pen, int x, int y) {
		if (b.GetPixel(x, y).ToArgb().Equals(pen.Color.ToArgb()))
			return;
		Stack<Point> s = new Stack<Point>();
		s.Push(new Point(x, y));
		while (s.Count > 0) {
			Point p = s.Pop();
			if (p.X < 0 || p.Y < 0 || p.X >= b.Width || p.Y >= b.Height)
				continue;
			if (b.GetPixel(p.X, p.Y).ToArgb().Equals(pen.Color.ToArgb()))
				continue;
			b.SetPixel(p.X, p.Y, pen.Color);
			s.Push(new Point(p.X + 1, p.Y));
			s.Push(new Point(p.X - 1, p.Y));
			s.Push(new Point(p.X, p.Y + 1));
			s.Push(new Point(p.X, p.Y - 1));
		}
	}

	/// <summary>
	/// Draws an empty rectangle to the screen.
	/// </summary>
	/// <param name="g">The graphics object to draw onto.</param>
	/// <param name="p">The pen that defines the color, width, and style of the rectange.</param>
	/// <param name="x1">The X value of a corner of the rectangle.</param>
	/// <param name="y1">The Y value of a corner of the rectangle.</param>
	/// <param name="x2">The X value of the opposite corner of the rectangle.</param>
	/// <param name="y2">The Y value of the opposite corner of the rectangle.</param>
	public static void DrawRect(this Graphics g, Pen p, int x1, int y1, int x2, int y2) {
		using (GraphicsPath gp = new GraphicsPath()) {
			gp.StartFigure();
			gp.AddLine(x1, y1, x2, y1);
			gp.AddLine(x2, y2, x2, y2);
			gp.AddLine(x2, y2, x1, y2);
			gp.CloseFigure();
			g.DrawPath(p, gp);
		}
	}

	/// <summary>
	/// Draws a filled rectangle to the screen
	/// </summary>
	/// <param name="g">The graphics object to draw onto.</param>
	/// <param name="b">The brush that determines the characteristics of the rectangle.</param>
	/// <param name="x1">The X value of a corner of the rectangle.</param>
	/// <param name="y1">The Y value of a corner of the rectangle.</param>
	/// <param name="x2">The X value of the opposite corner of the rectangle.</param>
	/// <param name="y2">The Y value of the opposite corner of the rectangle.</param>
	public static void FillRect(this Graphics g, Brush b, int x1, int y1, int x2, int y2) {
		using (GraphicsPath gp = new GraphicsPath()) {
			gp.StartFigure();
			gp.AddLine(x1, y1, x2, y1);
			gp.AddLine(x2, y2, x2, y2);
			gp.AddLine(x2, y2, x1, y2);
			gp.CloseFigure();
			g.FillPath(b, gp);
		}
	}

	public static void DrawCircle(this Graphics g, Pen p, int x, int y, int r) {
		g.DrawEllipse(p, x - r, y - r, r * 2, r * 2);
	}

	public static void FillCircle(this Graphics g, Brush b, int x, int y, int r) {
		g.FillEllipse(b, x - r, y - r, r * 2, r * 2);
	}
}